import * as mc from '@minecraft/server'

const ignoredGamemode = [mc.GameMode.creative, mc.GameMode.spectator]

function dmgEvent(e, s = false) {
	const entity = e.damagingEntity
	if (entity instanceof mc.Player && !ignoredGamemode.includes(entity.getGameMode()))
		cooldown(entity, true, s, e.hitEntity)
}

mc.world.afterEvents.entityHitBlock.subscribe(dmgEvent)
mc.world.afterEvents.entityHitEntity.subscribe(v => dmgEvent(v, true))
mc.world.afterEvents.playerSpawn.subscribe(e => {
	e.player.setDynamicProperty('cooldown', 0)
	e.player.setDynamicProperty('maxCooldown', 0)
})

const defaultCooldown = 6

const customCooldownItems = [

	[20/2, [/minecraft.(golden|diamond|netherite)_(hoe|axe)/, 'minecraft:trident']],
	[22/2, [/minecraft.iron_(hoe|axe)/]],
	[25/2, [/minecraft.(wooden|stone)_(hoe|axe)/]],
	[33.4/2, ['minecraft:mace']],
]

const customCooldownTags = [
	[12.5/2, [/minecraft.is_(sword|hoe|pickaxe)/]],
	[20/2, ['minecraft:is_shovel']],
	[22/2, ['minecraft:is_axe']],
]

const sweepingEffectTags = ['minecraft:is_sword']
const ignoredSweepingItems = ['minecraft:mace']

const customReduceCD = [[-100, ['vip']]]

mc.system.runInterval(() => {
	const players = mc.world.getPlayers({ excludeGameModes: ignoredGamemode })
	for (const player of players) cooldown(player)
}, 2)

const isString = s => typeof s === 'string'
const equals = (a, b) => (isString(a) ? a === b : a.test(b))
const applyPercents = (a, b) => Math.round(a - (a * b) / 100)

function cooldown(player, force = false, sound = false, hitEntity = void 0) {
	const itemStack = player.getComponent('equippable').getEquipment('Mainhand')
	const lastItem = player.getDynamicProperty('lastItem')
	const itemId = itemStack?.typeId
	let cooldown = player.getDynamicProperty('cooldown')
	let maxCooldown = player.getDynamicProperty('maxCooldown')

	if (itemId !== lastItem || force) {
		player.setDynamicProperty('lastItem', itemId)

		maxCooldown = defaultCooldown

		let itemTags = []

		if (itemStack) {
			itemTags = itemStack?.getTags() ?? []

			const matchedCd =
				customCooldownItems.find(([, n]) => n.some(v => equals(v, itemId)))?.[0] ||
				customCooldownTags.find(([, t]) => t.some(v => itemTags?.some(t2 => equals(v, t2))))?.[0]

			if (matchedCd) {
				const playerTags = player.getTags()
				const rcd = Math.min(customReduceCD.find(([, n]) => n.some(v => playerTags.includes(v)))?.[0] ?? 0, 100)
				maxCooldown = applyPercents(matchedCd, rcd)
			}
		}

		if (maxCooldown > defaultCooldown && !cooldown && sound && player.isFalling && !player.isInWater)
			player.dimension.playSound('crit', player.location)

		if (hitEntity && maxCooldown > defaultCooldown && !cooldown && !player.isFalling ) {
			const matchedSweepingEffect =
				ignoredSweepingItems.find(v => v !== itemId) && sweepingEffectTags.find(v => itemTags.includes(v))
 
			if (matchedSweepingEffect) hitEntity.dimension.spawnEntity('addon:sweeping', hitEntity.location)
		} 
		if (hitEntity && player.isSprinting) hitEntity.dimension.spawnEntity('addon:knockback', hitEntity.location)
		cooldown = maxCooldown

		player.setDynamicProperty('cooldown', maxCooldown)
		player.setDynamicProperty('maxCooldown', maxCooldown)
	}

	if (cooldown > 0) {
		player.removeEffect('weakness')
		player.addEffect('weakness', cooldown, {
			showParticles: false,
			amplifier: maxCooldown <= defaultCooldown ? 0 : Math.round((cooldown / maxCooldown) * 3),
		})
		cooldown--
		displayCooldown(player, cooldown, maxCooldown)
		player.setDynamicProperty('cooldown', cooldown)
	} else if (itemStack) {
		const targetEntities = player.getEntitiesFromViewDirection({ maxDistance: 5 })[0]
		if (!itemStack.isStackable && targetEntities) {
			if (targetEntities.entity.getComponent('health')) displayCooldown(player, cooldown, maxCooldown)
		}
	}
}

const icons = ['', '', '', '', '', '', '', '', '', '', '', '', '',  '']

function displayCooldown(player, currentTime, maxTime) {
	player.onScreenDisplay.setActionBar(`${icons[!maxTime ? 13 : 13 - Math.round((currentTime / maxTime) * 13)]}`)
}